/*
 * Copyright (c) 2003,2010 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef GENPOS_STRUCT_H
#define GENPOS_STRUCT_H

/* maximal size of a poster name */
#define GENPOS_MAX_POSTER_NAME 32

/* maximal size of trajectories */
#define GENPOS_MAX_TRAJECTORY_SIZE 1000

/* poster name */
typedef struct GENPOS_POSTER_NAME {
  char name[GENPOS_MAX_POSTER_NAME];
} GENPOS_POSTER_NAME;

/* configuration and velocity of a cart-like mobile robot */
/* GENPOS_CART_STATE */
typedef struct GENPOS_CART_STATE {
  double xRef;               /* position  */
  double yRef;               /* control point */
  double xRob;               /* position */
  double yRob;               /* robot center */
  double theta;              /* robot orientation*/
  double v;                  /* longitudinal linear velocity */
  double vt;                 /* linear velocity (transversal) */
  double w;                  /* angular velocity */
} GENPOS_CART_STATE;

/* configuration of a cart-like mobile robot */
/* GENPOS_CART_CONFIG */
typedef struct GENPOS_CART_CONFIG {
  double x;
  double y;
  double theta;
} GENPOS_CART_CONFIG;

/* Incertitudes sur x, y et theta */
/* GENPOS_CART_CONFIG_VAR */
typedef struct GENPOS_CART_CONFIG_VAR {
  double var[6];
} GENPOS_CART_CONFIG_VAR;

/* direction of motion */
typedef enum GENPOS_MOTION_DIRECTION {
  GENPOS_FORWARD_MOTION,
  GENPOS_BACKWARD_MOTION
} GENPOS_MOTION_DIRECTION;

typedef enum GENPOS_DATA_TYPE {
  GENPOS_POS_DATA,
  GENPOS_SPEED_DATA,
  GENPOS_POS_AND_SPEED_DATA
} GENPOS_DATA_TYPE;

/* GENPOS_CART_REF */
/* reference state in a trajectory tracking process */
typedef struct GENPOS_CART_REF {
  int numRef;                /* current reference number (incremented) */
  int tickPeriod;            /* Period of the ref update (0 if unknown) */
  GENPOS_DATA_TYPE dataType; /* Pertinent data following */
  GENPOS_MOTION_DIRECTION backFlag;  /* FORWARD or BACKWARD */
  double x;                  /* Abscisse ref point */
  double y;                  /* Ordonnee ref point */
  double theta;              /* Angle */
  double v;                  /* linear speed AT REF POINT (longitudinal) */
  double vt;                 /* transversal linear speed AT REF POINT */
  double w;                  /* angular speed AT REF POINT */
  double vmax;               /* linear speed max during servo */
  double wmax;               /* angular speed max during servo */
  double linAccelMax;        /* Accel lin max */
  double angAccelMax;        /* Accel ang max */
} GENPOS_CART_REF;

/* reference speed in a speed control process */
/* GENPOS_CART_SPEED */
typedef struct GENPOS_CART_SPEED {
  int numRef;                /* Number of the reference */
  int updatePeriod;          /* Number of loco period to reach the reference */
  double v;                  /* linear speed (longitudinal) */
  double vt;                 /* transversal linear speed */
  double w;                  /* angular speed */
  double vmax;
  double wmax;
  double linAccelMax;
  double angAccelMax;
} GENPOS_CART_SPEED;

/* control parameters of a tracking control law (PI) */
/* GENPOS_CART_CMD_PARAM */
typedef struct GENPOS_CART_CMD_PARAM {
  double distPoint;        /* distance control point robot center */
  double KpLongit;         /* longitudinal proportional gain */
  double KiLongit;         /* longitudinal integral gain */
  double KpTransv;         /* transversal proportional gain */
  double KiTransv;         /* transversal integral gain */
  double KpAng;            /* angular proportional gain */
  double KiAng;            /* angular integral gain */
  double longitErrorMax;   /* if dL > longitErrorMax, then SdL = 0 */
  double transvErrorMax;   /* if dT > transvErrorMax, then SdT = 0 */
  double longitMaxErrSum;  /* upper bound for SdL */
  double transvMaxErrSum;  /* upper bound for SdT */
  double angErrorMax;   /* if dT > transvErrorMax, alors SdT = 0 */
  double angMaxErrSum;  /* upper bound for SdT */
  double longitMaxGap;  /* if dL > longitMaxGap, tracking returns error*/
  double transvMaxGap;  /* if dl > transvMaxGap, tracking returns error*/
  double angMaxGap;     /* if dT > angMaxGap, tracking returns error*/
} GENPOS_CART_CMD_PARAM;

/* Parametre de l'asservissement en speed (PI) */
/* GENPOS_CART_CMD_SPEED_PARAM */
typedef struct GENPOS_CART_CMD_SPEED_PARAM {
  double Kp;         
  double Ki;         
  double Kd;         
  double Kff;        
  double maxErrorResetInt;   /* if dL > longitErrorMax, then SdL = 0 */
  double maxErrorSum;     /* Borne la somme des erreurs longit. SdL */
} GENPOS_CART_CMD_SPEED_PARAM;

typedef enum GENPOS_TRACK_MODE {
  GENPOS_NO_TRACKING,
  GENPOS_TRACK_POS,
  GENPOS_TRACK_CONFIG,
  GENPOS_TRACK_SPEED,
  GENPOS_TRACK_TRAILER
} GENPOS_TRACK_MODE;

/* A trajectory of GENPOS_CART_CONFIG elements that can
 * be tracked by a trajectory execution module */
typedef struct GENPOS_TRAJ_POINTS {
    /* This field starts at zero and is incremented when the trajectory is
     * updated */
    int numRef;
    /* Count of points actually in the trajectory */
    int nbPts;
    /* The absolute points in the trajectory */
    GENPOS_CART_CONFIG points[GENPOS_MAX_TRAJECTORY_SIZE];
} GENPOS_TRAJ_POINTS;

typedef enum GENPOS_TRAILER_TYPE {
  GENPOS_NO_TRAILER,
  GENPOS_TRAILER_OFF_AXLE,
  GENPOS_TRAILER_CENTERED_AXLE
} GENPOS_TRAILER_TYPE;

/* Informations about tracking */
/* GENPOS_TRACK */
typedef struct GENPOS_TRACK {
  GENPOS_TRACK_MODE trackMode;  
  long posterId;                  /* Id of poster */
  GENPOS_POSTER_NAME posterName;        /* name of poster */
} GENPOS_TRACK;



/* ========================= FOR THE TRAILER ============================ */

/* configuration of a cart trailer mobile robot */
/* GENPOS_TRCART_CONFIG */
typedef struct GENPOS_TRCART_CONFIG {
  GENPOS_CART_CONFIG robot;
  double phi;
} GENPOS_TRCART_CONFIG;

/* State of a cart trailer robot */
typedef struct GENPOS_TRCART_STATE {
  GENPOS_CART_STATE robot;
  double phi;
} GENPOS_TRCART_STATE;


/* reference state of a cart trailer mobile robot in a trajectory 
   tracking process */
/* GENPOS_TRCART_REF */
typedef struct GENPOS_TRCART_REF {
  GENPOS_CART_REF robot;              /* consigne du robot */
  double phi;                /* Angle remorque / robot */
} GENPOS_TRCART_REF;

/* geometric parameters of a trailer connection */
/* GENPOS_TRAILER_CONNECTION */
typedef struct GENPOS_TRAILER_CONNECTION {
  double l1; /* distance axe des roues robot, pt d'attache */ 
  double l2; /* distance axe des roues remorque, pt d'attache */
  double maxAngle; 
} GENPOS_TRAILER_CONNECTION;

/* GENPOS_TRCART_GEO_PARAM */
typedef struct GENPOS_TRCART_GEO_PARAM {
  GENPOS_TRAILER_CONNECTION trailerConnection;  /* trailer connection */
  double sensorAngOffset;  /* offset of the angular sensor */
} GENPOS_TRCART_GEO_PARAM;


#endif

